<?php
/* Driver template for the PHP_Haanga_rGenerator parser generator. (PHP port of LEMON)
*/

/**
 * This can be used to store both the string representation of
 * a token, and any useful meta-data associated with the token.
 *
 * meta-data should be stored as an array
 */
class Haanga_yyToken implements ArrayAccess
{
    public $string = '';
    public $metadata = array();

    function __construct($s, $m = array())
    {
        if ($s instanceof Haanga_yyToken) {
            $this->string = $s->string;
            $this->metadata = $s->metadata;
        } else {
            $this->string = (string) $s;
            if ($m instanceof Haanga_yyToken) {
                $this->metadata = $m->metadata;
            } elseif (is_array($m)) {
                $this->metadata = $m;
            }
        }
    }

    function __toString()
    {
        return $this->_string;
    }

    function offsetExists($offset)
    {
        return isset($this->metadata[$offset]);
    }

    function offsetGet($offset)
    {
        return $this->metadata[$offset];
    }

    function offsetSet($offset, $value)
    {
        if ($offset === null) {
            if (isset($value[0])) {
                $x = ($value instanceof Haanga_yyToken) ?
                    $value->metadata : $value;
                $this->metadata = array_merge($this->metadata, $x);
                return;
            }
            $offset = count($this->metadata);
        }
        if ($value === null) {
            return;
        }
        if ($value instanceof Haanga_yyToken) {
            if ($value->metadata) {
                $this->metadata[$offset] = $value->metadata;
            }
        } elseif ($value) {
            $this->metadata[$offset] = $value;
        }
    }

    function offsetUnset($offset)
    {
        unset($this->metadata[$offset]);
    }
}

/** The following structure represents a single element of the
 * parser's stack.  Information stored includes:
 *
 *   +  The state number for the parser at this level of the stack.
 *
 *   +  The value of the token stored at this level of the stack.
 *      (In other words, the "major" token.)
 *
 *   +  The semantic value stored at this level of the stack.  This is
 *      the information used by the action routines in the grammar.
 *      It is sometimes called the "minor" token.
 */
class Haanga_yyStackEntry
{
    public $stateno;       /* The state-number */
    public $major;         /* The major token value.  This is the code
                     ** number for the token at this stack level */
    public $minor; /* The user-supplied minor token value.  This
                     ** is the value of the token  */
};

// code external to the class is included here
#line 2 "Parser.y"

/*
  +---------------------------------------------------------------------------------+
  | Copyright (c) 2010 César Rodas and Menéame Comunicacions S.L.                   |
  +---------------------------------------------------------------------------------+
  | Redistribution and use in source and binary forms, with or without              |
  | modification, are permitted provided that the following conditions are met:     |
  | 1. Redistributions of source code must retain the above copyright               |
  |    notice, this list of conditions and the following disclaimer.                |
  |                                                                                 |
  | 2. Redistributions in binary form must reproduce the above copyright            |
  |    notice, this list of conditions and the following disclaimer in the          |
  |    documentation and/or other materials provided with the distribution.         |
  |                                                                                 |
  | 3. All advertising materials mentioning features or use of this software        |
  |    must display the following acknowledgement:                                  |
  |    This product includes software developed by César D. Rodas.                  |
  |                                                                                 |
  | 4. Neither the name of the César D. Rodas nor the                               |
  |    names of its contributors may be used to endorse or promote products         |
  |    derived from this software without specific prior written permission.        |
  |                                                                                 |
  | THIS SOFTWARE IS PROVIDED BY CÉSAR D. RODAS ''AS IS'' AND ANY                   |
  | EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       |
  | WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE          |
  | DISCLAIMED. IN NO EVENT SHALL CÉSAR D. RODAS BE LIABLE FOR ANY                  |
  | DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES      |
  | (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    |
  | LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND     |
  | ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT      |
  | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS   |
  | SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE                     |
  +---------------------------------------------------------------------------------+
  | Authors: César Rodas <crodas@php.net>                                           |
  +---------------------------------------------------------------------------------+
*/
#line 136 "Parser.php"

// declare_class is output here
#line 39 "Parser.y"
 class Haanga_Compiler_Parser #line 141 "Parser.php"
{
/* First off, code is included which follows the "include_class" declaration
** in the input file. */
#line 40 "Parser.y"

    protected $lex;
    protected $file;

    function __construct($lex, $file='')
    {
        $this->lex  = $lex;
        $this->file = $file;
    }

    function Error($text)
    {
        throw new Haanga_Compiler_Exception($text.' in '.$this->file.':'.$this->lex->getLine());
    }

#line 162 "Parser.php"

/* Next is all token values, as class constants
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
    const T_TAG_OPEN                     =  1;
    const T_NOT                          =  2;
    const T_AND                          =  3;
    const T_OR                           =  4;
    const T_EQ                           =  5;
    const T_NE                           =  6;
    const T_GT                           =  7;
    const T_GE                           =  8;
    const T_LT                           =  9;
    const T_LE                           = 10;
    const T_IN                           = 11;
    const T_PLUS                         = 12;
    const T_MINUS                        = 13;
    const T_CONCAT                       = 14;
    const T_TIMES                        = 15;
    const T_DIV                          = 16;
    const T_MOD                          = 17;
    const T_PIPE                         = 18;
    const T_BITWISE                      = 19;
    const T_HTML                         = 20;
    const T_COMMENT                      = 21;
    const T_PRINT_OPEN                   = 22;
    const T_PRINT_CLOSE                  = 23;
    const T_EXTENDS                      = 24;
    const T_TAG_CLOSE                    = 25;
    const T_INCLUDE                      = 26;
    const T_AUTOESCAPE                   = 27;
    const T_CUSTOM_END                   = 28;
    const T_CUSTOM_TAG                   = 29;
    const T_AS                           = 30;
    const T_CUSTOM_BLOCK                 = 31;
    const T_SPACEFULL                    = 32;
    const T_WITH                         = 33;
    const T_SET                          = 34;
    const T_ASSIGN                       = 35;
    const T_LOAD                         = 36;
    const T_FOR                          = 37;
    const T_COMMA                        = 38;
    const T_STEP                         = 39;
    const T_EMPTY                        = 40;
    const T_IF                           = 41;
    const T_ELSE                         = 42;
    const T_IFCHANGED                    = 43;
    const T_IFEQUAL                      = 44;
    const T_IFNOTEQUAL                   = 45;
    const T_BLOCK                        = 46;
    const T_FILTER                       = 47;
    const T_REGROUP                      = 48;
    const T_BY                           = 49;
    const T_COLON                        = 50;
    const T_TRUE                         = 51;
    const T_FALSE                        = 52;
    const T_STRING                       = 53;
    const T_INTL                         = 54;
    const T_RPARENT                      = 55;
    const T_LPARENT                      = 56;
    const T_OBJ                          = 57;
    const T_DOT                          = 58;
    const T_ALPHA                        = 59;
    const T_CLASS                        = 60;
    const T_BRACKETS_OPEN                = 61;
    const T_BRACKETS_CLOSE               = 62;
    const T_DOTDOT                       = 63;
    const T_NUMERIC                      = 64;
    const YY_NO_ACTION = 356;
    const YY_ACCEPT_ACTION = 355;
    const YY_ERROR_ACTION = 354;

/* Next are that tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < self::YYNSTATE                              Shift N.  That is,
**                                                        push the lookahead
**                                                        token onto the stack
**                                                        and goto state N.
**
**   self::YYNSTATE <= N < self::YYNSTATE+self::YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == self::YYNSTATE+self::YYNRULE                    A syntax error has occurred.
**
**   N == self::YYNSTATE+self::YYNRULE+1                  The parser accepts its
**                                                        input. (and concludes parsing)
**
**   N == self::YYNSTATE+self::YYNRULE+2                  No such action.  Denotes unused
**                                                        slots in the yy_action[] table.
**
** The action table is constructed as a single large static array $yy_action.
** Given state S and lookahead X, the action is computed as
**
**      self::$yy_action[self::$yy_shift_ofst[S] + X ]
**
** If the index value self::$yy_shift_ofst[S]+X is out of range or if the value
** self::$yy_lookahead[self::$yy_shift_ofst[S]+X] is not equal to X or if
** self::$yy_shift_ofst[S] is equal to self::YY_SHIFT_USE_DFLT, it means that
** the action is not in the table and that self::$yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the static $yy_reduce_ofst array is used in place of
** the static $yy_shift_ofst array and self::YY_REDUCE_USE_DFLT is used in place of
** self::YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  self::$yy_action        A single table containing all actions.
**  self::$yy_lookahead     A table containing the lookahead for each entry in
**                          yy_action.  Used to detect hash collisions.
**  self::$yy_shift_ofst    For each state, the offset into self::$yy_action for
**                          shifting terminals.
**  self::$yy_reduce_ofst   For each state, the offset into self::$yy_action for
**                          shifting non-terminals after a reduce.
**  self::$yy_default       Default action for each state.
*/
    const YY_SZ_ACTTAB = 1328;
static public $yy_action = array(
 /*     0 */    47,   57,   49,   65,  152,   39,  194,   40,  172,   64,
 /*    10 */    59,  195,   93,   67,  233,  155,  212,   29,  153,   41,
 /*    20 */    37,   36,   56,   53,   55,   47,   57,   49,   65,  177,
 /*    30 */    39,  247,   40,  172,   64,   59,   57,   93,   67,   25,
 /*    40 */    25,  178,   29,  202,   41,   37,   36,   56,   53,   55,
 /*    50 */    47,  161,   49,   65,  165,   39,  248,   40,  172,   64,
 /*    60 */    59,  224,   93,   67,  242,  126,  238,   29,  162,   41,
 /*    70 */    37,   36,   56,   53,   55,   47,  231,   49,   65,  182,
 /*    80 */    39,   97,   40,  172,   64,   59,  253,   93,   67,  109,
 /*    90 */   128,  201,   29,  189,   41,   37,   36,   56,   53,   55,
 /*   100 */    47,  231,   49,   65,  187,   39,  107,   40,  172,   64,
 /*   110 */    59,  204,   93,   67,  215,  124,  251,   29,  183,   41,
 /*   120 */    37,   36,   56,   53,   55,   47,  231,   49,   65,  169,
 /*   130 */    39,  191,   40,  172,   64,   59,  237,   93,   67,  196,
 /*   140 */   139,  111,   29,  171,   41,   37,   36,   56,   53,   55,
 /*   150 */    47,  231,   49,   65,  158,   39,  199,   40,  172,   64,
 /*   160 */    59,   57,   93,   67,  108,  127,  104,   29,   95,   41,
 /*   170 */    37,   36,   56,   53,   55,   47,  231,   49,   65,  151,
 /*   180 */    39,  197,   40,  172,   64,   59,   57,   93,   67,  106,
 /*   190 */   135,  193,   29,  239,   41,   37,   36,   56,   53,   55,
 /*   200 */    47,  231,   49,   65,   92,   39,  100,   40,  172,   64,
 /*   210 */    59,  244,   93,   67,  210,  125,  110,   29,  115,   41,
 /*   220 */    37,   36,   56,   53,   55,   47,  231,   49,   65,  160,
 /*   230 */    39,  243,   40,  172,   64,   59,  236,   93,   67,  252,
 /*   240 */   130,  209,   29,   50,   41,   37,   36,   56,   53,   55,
 /*   250 */    47,  231,   49,   65,  184,   39,  229,   40,  172,   64,
 /*   260 */    59,  246,   93,   67,  102,  131,  225,   29,  232,   41,
 /*   270 */    37,   36,   56,   53,   55,   47,  231,   49,   65,  173,
 /*   280 */    39,  241,   40,  172,   64,   59,   83,   93,   67,   88,
 /*   290 */   133,   87,   29,  179,   41,   37,   36,   56,   53,   55,
 /*   300 */    47,  231,   49,   65,  188,   39,  184,   40,  172,   64,
 /*   310 */    59,   90,   93,   67,   89,  129,   85,   29,  184,   41,
 /*   320 */    37,   36,   56,   53,   55,   47,  231,   49,   65,  168,
 /*   330 */    39,   84,   40,  172,   64,   59,   86,   93,   67,  355,
 /*   340 */    78,   79,   29,  184,   41,   37,   36,   56,   53,   55,
 /*   350 */    47,   76,   49,   65,  170,   39,   81,   40,  172,   64,
 /*   360 */    59,   74,   93,   67,   75,   71,   77,   29,  184,   41,
 /*   370 */    37,   36,   56,   53,   55,   47,   80,   49,   65,  181,
 /*   380 */    39,   73,   40,  172,   64,   59,  184,   93,   67,   82,
 /*   390 */    72,   70,   29,  184,   41,   37,   36,   56,   53,   55,
 /*   400 */    47,   91,   49,   65,   58,   39,   69,   40,  172,   64,
 /*   410 */    59,  184,   93,   67,  184,  184,  184,   29,  184,   41,
 /*   420 */    37,   36,   56,   53,   55,   47,  184,   49,   65,  185,
 /*   430 */    39,  184,   40,  172,   64,   59,  184,   93,   67,  184,
 /*   440 */   184,  184,   29,  184,   41,   37,   36,   56,   53,   55,
 /*   450 */    47,  184,   49,   65,  149,   39,  184,   40,  172,   64,
 /*   460 */    59,  184,   93,   67,  184,  184,  184,   29,  184,   41,
 /*   470 */    37,   36,   56,   53,   55,   47,  184,   49,   65,  175,
 /*   480 */    39,  184,   40,  172,   64,   59,  184,   93,   67,  184,
 /*   490 */   184,  184,   29,   57,   41,   37,   36,   56,   53,   55,
 /*   500 */    30,   31,   32,   32,   32,   32,   32,   32,   32,   27,
 /*   510 */    27,   27,   26,   26,   26,   25,   25,   26,   26,   26,
 /*   520 */    25,   25,  113,   47,   66,   49,   65,  156,   39,  184,
 /*   530 */    40,  172,   64,   59,  184,   93,   67,  184,  184,  184,
 /*   540 */    29,  184,   41,   37,   36,   56,   53,   55,   47,  184,
 /*   550 */    49,   65,  180,   39,  184,   40,  172,   64,   59,  184,
 /*   560 */    93,   67,  184,  184,  184,   29,  184,   41,   37,   36,
 /*   570 */    56,   53,   55,   30,   31,   32,   32,   32,   32,   32,
 /*   580 */    32,   32,   27,   27,   27,   26,   26,   26,   25,   25,
 /*   590 */    47,  184,   49,   65,  132,   39,  184,   40,  172,   64,
 /*   600 */    59,  184,   93,   67,  257,  231,  143,   29,   45,   41,
 /*   610 */    37,   36,   56,   53,   55,  140,  140,  123,  141,   48,
 /*   620 */   184,  351,  184,  148,  164,  250,  198,  263,  231,   30,
 /*   630 */    31,   32,   32,   32,   32,   32,   32,   32,   27,   27,
 /*   640 */    27,   26,   26,   26,   25,   25,   31,   32,   32,   32,
 /*   650 */    32,   32,   32,   32,   27,   27,   27,   26,   26,   26,
 /*   660 */    25,   25,   32,   32,   32,   32,   32,   32,   32,   27,
 /*   670 */    27,   27,   26,   26,   26,   25,   25,  167,  245,  223,
 /*   680 */    45,  223,  220,  184,  220,  174,  184,  140,  140,  234,
 /*   690 */   141,   48,  144,  220,   61,  220,  132,   28,  122,  220,
 /*   700 */   259,   15,   46,  132,  249,  262,  263,  231,  167,  223,
 /*   710 */   220,   94,  230,  263,  231,  227,  227,  233,  155,  184,
 /*   720 */   203,  200,   54,  230,  220,   51,  220,  226,  219,  184,
 /*   730 */   154,  208,  207,  205,  206,  211,  217,  218,  216,  147,
 /*   740 */   167,  220,  240,    4,   96,  184,  261,  261,  233,  155,
 /*   750 */   132,   24,  213,  184,  230,  184,  220,   62,  220,  219,
 /*   760 */   263,  231,  203,  200,   54,  184,  264,  184,  184,  184,
 /*   770 */   184,  167,  184,  220,  254,  184,  184,  184,  227,  227,
 /*   780 */   233,  155,  184,   99,  138,   42,  230,  220,  228,  220,
 /*   790 */   184,  219,  184,  222,  167,  231,   46,  112,  140,  140,
 /*   800 */   184,  141,   48,  167,  220,  174,  190,  184,  184,  227,
 /*   810 */   227,  233,  155,  184,  184,   98,  132,  230,  116,  220,
 /*   820 */   259,  220,  219,  105,  249,  262,  263,  231,   46,  140,
 /*   830 */   140,  184,  141,   48,  184,  167,  220,  184,  184,  184,
 /*   840 */   184,  227,  227,  233,  155,  219,  184,  101,  184,  230,
 /*   850 */   174,  220,  184,  220,  219,  140,  140,  184,  141,   48,
 /*   860 */   221,  132,  221,  117,  167,  259,  184,  167,  220,  249,
 /*   870 */   262,  263,  231,  227,  227,  233,  155,  254,  184,  103,
 /*   880 */   220,  230,  220,  220,  184,  220,  219,  138,   43,  184,
 /*   890 */   221,  228,  184,  167,   63,  184,  222,  220,  231,  220,
 /*   900 */   220,  220,  184,  184,  184,  227,  227,  233,  155,  220,
 /*   910 */   230,  220,  184,  230,  235,  219,  220,  184,  219,  167,
 /*   920 */   184,  140,  140,  184,  141,   48,  220,  134,  184,  230,
 /*   930 */   184,  227,  227,  233,  155,  220,  176,  220,  231,  230,
 /*   940 */   184,  184,  184,  184,  219,  184,  140,  140,  184,  141,
 /*   950 */    48,  184,  220,  184,  184,  137,  174,  261,  261,  233,
 /*   960 */   155,  184,  192,  184,  198,  230,  231,  132,   13,  260,
 /*   970 */   219,  259,  254,  184,  184,  249,  262,  263,  231,  174,
 /*   980 */   184,  184,  138,   44,   22,  142,  228,  203,  200,   54,
 /*   990 */   132,  222,  121,  231,  259,  184,  132,  174,  249,  262,
 /*  1000 */   263,  231,    9,  203,  200,   54,  263,  231,  132,  184,
 /*  1010 */   118,  184,  259,  184,  184,  174,  249,  262,  263,  231,
 /*  1020 */   184,  203,  200,   54,   38,  184,  132,  174,  145,  184,
 /*  1030 */   259,  184,  184,  184,  249,  262,  263,  231,  132,  184,
 /*  1040 */   120,  174,  259,  184,  184,  184,  249,  262,  263,  231,
 /*  1050 */   184,   68,  132,  184,  119,  184,  259,  184,  184,  184,
 /*  1060 */   249,  262,  263,  231,  184,  174,   33,  184,  184,  184,
 /*  1070 */   140,  140,  184,  141,   48,  184,  132,  184,  136,  174,
 /*  1080 */   259,  184,  184,  184,  249,  262,  263,  231,  140,  140,
 /*  1090 */   132,  141,   48,  146,  259,  184,  184,  174,  163,  262,
 /*  1100 */   263,  231,   52,  184,  132,   60,  184,  184,  132,  174,
 /*  1110 */   184,  184,  259,  184,  263,  231,   35,  262,  263,  231,
 /*  1120 */   132,  184,  184,  174,  259,  114,  184,  184,  186,  262,
 /*  1130 */   263,  231,  140,  140,  132,  141,   48,  184,  259,  184,
 /*  1140 */   184,  184,   34,  262,  263,  231,  214,  184,  140,  140,
 /*  1150 */   184,  141,   48,  184,  184,  184,  159,  140,  140,  184,
 /*  1160 */   141,   48,  184,  184,  184,  184,  138,  184,  184,  184,
 /*  1170 */   228,  184,  184,  166,  256,  222,  184,  231,  140,  140,
 /*  1180 */   184,  141,   48,  138,  138,  184,   21,  228,  228,  258,
 /*  1190 */   255,  184,  222,  222,  231,  231,  184,  184,  184,  138,
 /*  1200 */   138,  184,  150,  228,  228,  203,  200,   54,  222,  222,
 /*  1210 */   231,  231,  138,    3,  184,  137,  228,  184,   20,  184,
 /*  1220 */   184,  222,  157,  231,  198,   23,  231,  184,  184,  184,
 /*  1230 */   184,  184,  203,  200,   54,   18,  184,  203,  200,   54,
 /*  1240 */    11,  184,  184,  184,  203,  200,   54,    1,  140,  140,
 /*  1250 */   184,  141,   48,    2,  203,  200,   54,   17,  184,  203,
 /*  1260 */   200,   54,   10,  184,  184,  184,  203,  200,   54,   14,
 /*  1270 */   184,  184,  203,  200,   54,    5,  203,  200,   54,   16,
 /*  1280 */   184,  203,  200,   54,    6,  184,  184,  184,  203,  200,
 /*  1290 */    54,   19,  184,  184,  203,  200,   54,    7,  203,  200,
 /*  1300 */    54,   12,  184,  203,  200,   54,    8,  184,  184,  184,
 /*  1310 */   203,  200,   54,  184,  184,  184,  203,  200,   54,  184,
 /*  1320 */   203,  200,   54,  184,  184,  203,  200,   54,
    );
    static public $yy_lookahead = array(
 /*     0 */    24,   18,   26,   27,   28,   29,   23,   31,   32,   33,
 /*    10 */    34,   25,   36,   37,   53,   54,   25,   41,   42,   43,
 /*    20 */    44,   45,   46,   47,   48,   24,   18,   26,   27,   28,
 /*    30 */    29,   25,   31,   32,   33,   34,   18,   36,   37,   18,
 /*    40 */    19,   40,   41,   25,   43,   44,   45,   46,   47,   48,
 /*    50 */    24,   53,   26,   27,   28,   29,   25,   31,   32,   33,
 /*    60 */    34,   25,   36,   37,   25,   81,   25,   41,   42,   43,
 /*    70 */    44,   45,   46,   47,   48,   24,   92,   26,   27,   28,
 /*    80 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   25,
 /*    90 */    81,   25,   41,   42,   43,   44,   45,   46,   47,   48,
 /*   100 */    24,   92,   26,   27,   28,   29,   25,   31,   32,   33,
 /*   110 */    34,   25,   36,   37,   25,   81,   25,   41,   42,   43,
 /*   120 */    44,   45,   46,   47,   48,   24,   92,   26,   27,   28,
 /*   130 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   25,
 /*   140 */    81,   25,   41,   42,   43,   44,   45,   46,   47,   48,
 /*   150 */    24,   92,   26,   27,   28,   29,   25,   31,   32,   33,
 /*   160 */    34,   18,   36,   37,   25,   81,   25,   41,   25,   43,
 /*   170 */    44,   45,   46,   47,   48,   24,   92,   26,   27,   28,
 /*   180 */    29,   25,   31,   32,   33,   34,   18,   36,   37,   25,
 /*   190 */    81,   25,   41,   25,   43,   44,   45,   46,   47,   48,
 /*   200 */    24,   92,   26,   27,   28,   29,   25,   31,   32,   33,
 /*   210 */    34,   25,   36,   37,   25,   81,   25,   41,   25,   43,
 /*   220 */    44,   45,   46,   47,   48,   24,   92,   26,   27,   28,
 /*   230 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   25,
 /*   240 */    81,   64,   41,   63,   43,   44,   45,   46,   47,   48,
 /*   250 */    24,   92,   26,   27,   28,   29,   62,   31,   32,   33,
 /*   260 */    34,   25,   36,   37,   25,   81,   68,   41,   55,   43,
 /*   270 */    44,   45,   46,   47,   48,   24,   92,   26,   27,   28,
 /*   280 */    29,   85,   31,   32,   33,   34,   67,   36,   37,   67,
 /*   290 */    81,   67,   41,   90,   43,   44,   45,   46,   47,   48,
 /*   300 */    24,   92,   26,   27,   28,   29,   93,   31,   32,   33,
 /*   310 */    34,   67,   36,   37,   67,   81,   67,   41,   93,   43,
 /*   320 */    44,   45,   46,   47,   48,   24,   92,   26,   27,   28,
 /*   330 */    29,   67,   31,   32,   33,   34,   67,   36,   37,   66,
 /*   340 */    67,   67,   41,   93,   43,   44,   45,   46,   47,   48,
 /*   350 */    24,   67,   26,   27,   28,   29,   67,   31,   32,   33,
 /*   360 */    34,   67,   36,   37,   67,   67,   67,   41,   93,   43,
 /*   370 */    44,   45,   46,   47,   48,   24,   67,   26,   27,   28,
 /*   380 */    29,   67,   31,   32,   33,   34,   93,   36,   37,   67,
 /*   390 */    67,   67,   41,   93,   43,   44,   45,   46,   47,   48,
 /*   400 */    24,   67,   26,   27,   28,   29,   67,   31,   32,   33,
 /*   410 */    34,   93,   36,   37,   93,   93,   93,   41,   93,   43,
 /*   420 */    44,   45,   46,   47,   48,   24,   93,   26,   27,   28,
 /*   430 */    29,   93,   31,   32,   33,   34,   93,   36,   37,   93,
 /*   440 */    93,   93,   41,   93,   43,   44,   45,   46,   47,   48,
 /*   450 */    24,   93,   26,   27,   28,   29,   93,   31,   32,   33,
 /*   460 */    34,   93,   36,   37,   93,   93,   93,   41,   93,   43,
 /*   470 */    44,   45,   46,   47,   48,   24,   93,   26,   27,   28,
 /*   480 */    29,   93,   31,   32,   33,   34,   93,   36,   37,   93,
 /*   490 */    93,   93,   41,   18,   43,   44,   45,   46,   47,   48,
 /*   500 */     3,    4,    5,    6,    7,    8,    9,   10,   11,   12,
 /*   510 */    13,   14,   15,   16,   17,   18,   19,   15,   16,   17,
 /*   520 */    18,   19,   25,   24,   49,   26,   27,   28,   29,   93,
 /*   530 */    31,   32,   33,   34,   93,   36,   37,   93,   93,   93,
 /*   540 */    41,   93,   43,   44,   45,   46,   47,   48,   24,   93,
 /*   550 */    26,   27,   28,   29,   93,   31,   32,   33,   34,   93,
 /*   560 */    36,   37,   93,   93,   93,   41,   93,   43,   44,   45,
 /*   570 */    46,   47,   48,    3,    4,    5,    6,    7,    8,    9,
 /*   580 */    10,   11,   12,   13,   14,   15,   16,   17,   18,   19,
 /*   590 */    24,   93,   26,   27,   81,   29,   93,   31,   32,   33,
 /*   600 */    34,   93,   36,   37,   91,   92,   70,   41,   50,   43,
 /*   610 */    44,   45,   46,   47,   48,   57,   58,   81,   60,   61,
 /*   620 */    93,   63,   93,   87,   88,   55,   90,   91,   92,    3,
 /*   630 */     4,    5,    6,    7,    8,    9,   10,   11,   12,   13,
 /*   640 */    14,   15,   16,   17,   18,   19,    4,    5,    6,    7,
 /*   650 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*   660 */    18,   19,    5,    6,    7,    8,    9,   10,   11,   12,
 /*   670 */    13,   14,   15,   16,   17,   18,   19,   13,   25,   29,
 /*   680 */    50,   31,   29,   93,   31,   70,   93,   57,   58,   25,
 /*   690 */    60,   61,   70,   29,   30,   31,   81,    2,   83,   46,
 /*   700 */    85,    1,   38,   81,   89,   90,   91,   92,   13,   59,
 /*   710 */    46,   25,   59,   91,   92,   51,   52,   53,   54,   93,
 /*   720 */    20,   21,   22,   59,   29,   39,   31,   69,   64,   93,
 /*   730 */    72,   73,   74,   75,   76,   77,   78,   79,   80,   70,
 /*   740 */    13,   46,   84,    1,   86,   93,   51,   52,   53,   54,
 /*   750 */    81,   56,   25,   93,   59,   93,   29,   30,   31,   64,
 /*   760 */    91,   92,   20,   21,   22,   93,   25,   93,   93,   93,
 /*   770 */    93,   13,   93,   46,   71,   93,   93,   93,   51,   52,
 /*   780 */    53,   54,   93,   25,   81,   82,   59,   29,   85,   31,
 /*   790 */    93,   64,   93,   90,   13,   92,   38,   25,   57,   58,
 /*   800 */    93,   60,   61,   13,   46,   70,   25,   93,   93,   51,
 /*   810 */    52,   53,   54,   93,   93,   25,   81,   59,   83,   29,
 /*   820 */    85,   31,   64,   25,   89,   90,   91,   92,   38,   57,
 /*   830 */    58,   93,   60,   61,   93,   13,   46,   93,   93,   93,
 /*   840 */    93,   51,   52,   53,   54,   64,   93,   25,   93,   59,
 /*   850 */    70,   29,   93,   31,   64,   57,   58,   93,   60,   61,
 /*   860 */    29,   81,   31,   83,   13,   85,   93,   13,   46,   89,
 /*   870 */    90,   91,   92,   51,   52,   53,   54,   71,   93,   25,
 /*   880 */    29,   59,   31,   29,   93,   31,   64,   81,   82,   93,
 /*   890 */    59,   85,   93,   13,   30,   93,   90,   46,   92,   29,
 /*   900 */    46,   31,   93,   93,   93,   51,   52,   53,   54,   29,
 /*   910 */    59,   31,   93,   59,   25,   64,   46,   93,   64,   13,
 /*   920 */    93,   57,   58,   93,   60,   61,   46,   81,   93,   59,
 /*   930 */    93,   51,   52,   53,   54,   29,   90,   31,   92,   59,
 /*   940 */    93,   93,   93,   93,   64,   93,   57,   58,   93,   60,
 /*   950 */    61,   93,   46,   93,   93,   81,   70,   51,   52,   53,
 /*   960 */    54,   93,   88,   93,   90,   59,   92,   81,    1,   83,
 /*   970 */    64,   85,   71,   93,   93,   89,   90,   91,   92,   70,
 /*   980 */    93,   93,   81,   82,    1,   70,   85,   20,   21,   22,
 /*   990 */    81,   90,   83,   92,   85,   93,   81,   70,   89,   90,
 /*  1000 */    91,   92,    1,   20,   21,   22,   91,   92,   81,   93,
 /*  1010 */    83,   93,   85,   93,   93,   70,   89,   90,   91,   92,
 /*  1020 */    93,   20,   21,   22,   11,   93,   81,   70,   83,   93,
 /*  1030 */    85,   93,   93,   93,   89,   90,   91,   92,   81,   93,
 /*  1040 */    83,   70,   85,   93,   93,   93,   89,   90,   91,   92,
 /*  1050 */    93,   38,   81,   93,   83,   93,   85,   93,   93,   93,
 /*  1060 */    89,   90,   91,   92,   93,   70,   35,   93,   93,   93,
 /*  1070 */    57,   58,   93,   60,   61,   93,   81,   93,   83,   70,
 /*  1080 */    85,   93,   93,   93,   89,   90,   91,   92,   57,   58,
 /*  1090 */    81,   60,   61,   70,   85,   93,   93,   70,   89,   90,
 /*  1100 */    91,   92,   11,   93,   81,   30,   93,   93,   81,   70,
 /*  1110 */    93,   93,   85,   93,   91,   92,   89,   90,   91,   92,
 /*  1120 */    81,   93,   93,   70,   85,   25,   93,   93,   89,   90,
 /*  1130 */    91,   92,   57,   58,   81,   60,   61,   93,   85,   93,
 /*  1140 */    93,   93,   89,   90,   91,   92,   25,   93,   57,   58,
 /*  1150 */    93,   60,   61,   93,   93,   93,   71,   57,   58,   93,
 /*  1160 */    60,   61,   93,   93,   93,   93,   81,   93,   93,   93,
 /*  1170 */    85,   93,   93,   71,   71,   90,   93,   92,   57,   58,
 /*  1180 */    93,   60,   61,   81,   81,   93,    1,   85,   85,   71,
 /*  1190 */    71,   93,   90,   90,   92,   92,   93,   93,   93,   81,
 /*  1200 */    81,   93,   71,   85,   85,   20,   21,   22,   90,   90,
 /*  1210 */    92,   92,   81,    1,   93,   81,   85,   93,    1,   93,
 /*  1220 */    93,   90,   88,   92,   90,    1,   92,   93,   93,   93,
 /*  1230 */    93,   93,   20,   21,   22,    1,   93,   20,   21,   22,
 /*  1240 */     1,   93,   93,   93,   20,   21,   22,    1,   57,   58,
 /*  1250 */    93,   60,   61,    1,   20,   21,   22,    1,   93,   20,
 /*  1260 */    21,   22,    1,   93,   93,   93,   20,   21,   22,    1,
 /*  1270 */    93,   93,   20,   21,   22,    1,   20,   21,   22,    1,
 /*  1280 */    93,   20,   21,   22,    1,   93,   93,   93,   20,   21,
 /*  1290 */    22,    1,   93,   93,   20,   21,   22,    1,   20,   21,
 /*  1300 */    22,    1,   93,   20,   21,   22,    1,   93,   93,   93,
 /*  1310 */    20,   21,   22,   93,   93,   93,   20,   21,   22,   93,
 /*  1320 */    20,   21,   22,   93,   93,   20,   21,   22,
);
    const YY_SHIFT_USE_DFLT = -40;
    const YY_SHIFT_MAX = 189;
    static public $yy_shift_ofst = array(
 /*     0 */   -40,  101,   51,  -24,   76,    1,   26,  451,  499,  524,
 /*    10 */   351,  426,  201,  226,  176,  151,  126,  276,  376,  326,
 /*    20 */   401,  301,  251,  566,  695,  695,  695,  695,  695,  695,
 /*    30 */   695,  695,  695,  695,  906,  906,  906,  906,  851,  727,
 /*    40 */   822,  854,  664,  758,  790,  880,  880,  880,  880,  880,
 /*    50 */   851,  851,  870,  870,  870,  870,  851,  870,  653,  870,
 /*    60 */   870,  870,  870,  870,  870,  870,  870,  870,  870, 1296,
 /*    70 */  1283, 1278, 1274, 1290, 1305, 1300,  967, 1234, 1224, 1239,
 /*    80 */  1246, 1256, 1217, 1212,  742,  700,  983, 1252, 1185, 1001,
 /*    90 */  1261, 1268,  781,  -39,  -40,  -40,  -40,  -40,  -40,  -40,
 /*   100 */   -40,  -40,  -40,  -40,  -40,  -40,  -40,  -40,  -40,  -40,
 /*   110 */   -40,  -40,  -40,  -40,  -40,  -40,  570,  497,  626,  626,
 /*   120 */   642,  657,  657,  558, 1013, 1100,  798, 1031,  889, 1075,
 /*   130 */  1121, 1091,  630,  864,  772,  741,  502, 1191, 1191, 1191,
 /*   140 */   650,  831,  -17,  168,  143,   21,  475,   18,  686,    6,
 /*   150 */    -9,  -14,   31,  239,  189,   -2,  186,  181,  156,   36,
 /*   160 */   166,  213,  191,  193,  180,  236,  194,  177,  206,  211,
 /*   170 */   214,  139,   64,   61,    8,   41,   56,   86,  116,  114,
 /*   180 */   111,   91,  106,  164,   89,  131,   81,   39,   66,  141,
);
    const YY_REDUCE_USE_DFLT = -17;
    const YY_REDUCE_MAX = 115;
    static public $yy_reduce_ofst = array(
 /*     0 */   273,  658,  658,  658,  658,  658,  658,  658,  658,  658,
 /*    10 */   658,  658,  658,  658,  658,  658,  658,  658,  658,  658,
 /*    20 */   658,  658,  658,  658,  735,  886,  945,  995,  927,  780,
 /*    30 */   957,  909,  615,  971, 1039, 1009, 1027, 1053,  536,  703,
 /*    40 */   901,  806, 1119, 1119, 1119, 1118, 1103, 1085, 1102, 1131,
 /*    50 */   874, 1134,  669,  622,  915, 1023,  846,  513,  109,   84,
 /*    60 */    59,    9,  159,  134,  209,  -16,  234,   34,  184,  198,
 /*    70 */   198,  198,  198,  198,  198,  198,  198,  198,  198,  198,
 /*    80 */   198,  198,  198,  198,  198,  198,  198,  198,  198,  198,
 /*    90 */   198,  198,  203,  196,  322,  298,  323,  334,  297,  224,
 /*   100 */   249,  269,  274,  264,  289,  284,  222,  219,  247,  244,
 /*   110 */   294,  339,  299,  309,  314,  324,
);
    static public $yyExpectedTokens = array(
        /* 0 */ array(),
        /* 1 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 2 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 3 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 4 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 5 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 40, 41, 43, 44, 45, 46, 47, 48, ),
        /* 6 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 7 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 8 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 9 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 10 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 11 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 12 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 13 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 14 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 15 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 16 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 17 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 18 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 19 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 20 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 21 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 22 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 23 */ array(24, 26, 27, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 24 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 25 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 26 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 27 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 28 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 29 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 30 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 31 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 32 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 33 */ array(2, 13, 29, 31, 46, 51, 52, 53, 54, 56, 59, 64, ),
        /* 34 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 35 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 36 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 37 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 38 */ array(13, 29, 31, 46, 59, 64, ),
        /* 39 */ array(13, 25, 29, 30, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 40 */ array(13, 25, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 41 */ array(13, 25, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 42 */ array(13, 25, 29, 30, 31, 38, 46, 51, 52, 53, 54, 59, 64, ),
        /* 43 */ array(13, 25, 29, 31, 38, 46, 51, 52, 53, 54, 59, 64, ),
        /* 44 */ array(13, 25, 29, 31, 38, 46, 51, 52, 53, 54, 59, 64, ),
        /* 45 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 46 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 47 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 48 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 49 */ array(13, 29, 31, 46, 51, 52, 53, 54, 59, 64, ),
        /* 50 */ array(13, 29, 31, 46, 59, 64, ),
        /* 51 */ array(13, 29, 31, 46, 59, 64, ),
        /* 52 */ array(29, 31, 46, 59, ),
        /* 53 */ array(29, 31, 46, 59, ),
        /* 54 */ array(29, 31, 46, 59, ),
        /* 55 */ array(29, 31, 46, 59, ),
        /* 56 */ array(13, 29, 31, 46, 59, 64, ),
        /* 57 */ array(29, 31, 46, 59, ),
        /* 58 */ array(25, 29, 31, 46, 59, ),
        /* 59 */ array(29, 31, 46, 59, ),
        /* 60 */ array(29, 31, 46, 59, ),
        /* 61 */ array(29, 31, 46, 59, ),
        /* 62 */ array(29, 31, 46, 59, ),
        /* 63 */ array(29, 31, 46, 59, ),
        /* 64 */ array(29, 31, 46, 59, ),
        /* 65 */ array(29, 31, 46, 59, ),
        /* 66 */ array(29, 31, 46, 59, ),
        /* 67 */ array(29, 31, 46, 59, ),
        /* 68 */ array(29, 31, 46, 59, ),
        /* 69 */ array(1, 20, 21, 22, ),
        /* 70 */ array(1, 20, 21, 22, ),
        /* 71 */ array(1, 20, 21, 22, ),
        /* 72 */ array(1, 20, 21, 22, ),
        /* 73 */ array(1, 20, 21, 22, ),
        /* 74 */ array(1, 20, 21, 22, ),
        /* 75 */ array(1, 20, 21, 22, ),
        /* 76 */ array(1, 20, 21, 22, ),
        /* 77 */ array(1, 20, 21, 22, ),
        /* 78 */ array(1, 20, 21, 22, ),
        /* 79 */ array(1, 20, 21, 22, ),
        /* 80 */ array(1, 20, 21, 22, ),
        /* 81 */ array(1, 20, 21, 22, ),
        /* 82 */ array(1, 20, 21, 22, ),
        /* 83 */ array(1, 20, 21, 22, ),
        /* 84 */ array(1, 20, 21, 22, ),
        /* 85 */ array(1, 20, 21, 22, ),
        /* 86 */ array(1, 20, 21, 22, ),
        /* 87 */ array(1, 20, 21, 22, ),
        /* 88 */ array(1, 20, 21, 22, ),
        /* 89 */ array(1, 20, 21, 22, ),
        /* 90 */ array(1, 20, 21, 22, ),
        /* 91 */ array(1, 20, 21, 22, ),
        /* 92 */ array(13, 25, 64, ),
        /* 93 */ array(53, 54, ),
        /* 94 */ array(),
        /* 95 */ array(),
        /* 96 */ array(),
        /* 97 */ array(),
        /* 98 */ array(),
        /* 99 */ array(),
        /* 100 */ array(),
        /* 101 */ array(),
        /* 102 */ array(),
        /* 103 */ array(),
        /* 104 */ array(),
        /* 105 */ array(),
        /* 106 */ array(),
        /* 107 */ array(),
        /* 108 */ array(),
        /* 109 */ array(),
        /* 110 */ array(),
        /* 111 */ array(),
        /* 112 */ array(),
        /* 113 */ array(),
        /* 114 */ array(),
        /* 115 */ array(),
        /* 116 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 55, ),
        /* 117 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 25, ),
        /* 118 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 119 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 120 */ array(4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 121 */ array(5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 122 */ array(5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 123 */ array(50, 57, 58, 60, 61, 63, ),
        /* 124 */ array(11, 38, 57, 58, 60, 61, ),
        /* 125 */ array(25, 57, 58, 60, 61, ),
        /* 126 */ array(25, 57, 58, 60, 61, ),
        /* 127 */ array(35, 57, 58, 60, 61, ),
        /* 128 */ array(25, 57, 58, 60, 61, ),
        /* 129 */ array(30, 57, 58, 60, 61, ),
        /* 130 */ array(25, 57, 58, 60, 61, ),
        /* 131 */ array(11, 57, 58, 60, 61, ),
        /* 132 */ array(50, 57, 58, 60, 61, ),
        /* 133 */ array(30, 57, 58, 60, 61, ),
        /* 134 */ array(25, 57, 58, 60, 61, ),
        /* 135 */ array(25, 57, 58, 60, 61, ),
        /* 136 */ array(15, 16, 17, 18, 19, ),
        /* 137 */ array(57, 58, 60, 61, ),
        /* 138 */ array(57, 58, 60, 61, ),
        /* 139 */ array(57, 58, 60, 61, ),
        /* 140 */ array(29, 31, 59, ),
        /* 141 */ array(29, 31, 59, ),
        /* 142 */ array(18, 23, ),
        /* 143 */ array(18, 25, ),
        /* 144 */ array(18, 25, ),
        /* 145 */ array(18, 19, ),
        /* 146 */ array(18, 49, ),
        /* 147 */ array(18, 25, ),
        /* 148 */ array(25, 39, ),
        /* 149 */ array(25, ),
        /* 150 */ array(25, ),
        /* 151 */ array(25, ),
        /* 152 */ array(25, ),
        /* 153 */ array(25, ),
        /* 154 */ array(25, ),
        /* 155 */ array(53, ),
        /* 156 */ array(25, ),
        /* 157 */ array(25, ),
        /* 158 */ array(25, ),
        /* 159 */ array(25, ),
        /* 160 */ array(25, ),
        /* 161 */ array(55, ),
        /* 162 */ array(25, ),
        /* 163 */ array(25, ),
        /* 164 */ array(63, ),
        /* 165 */ array(25, ),
        /* 166 */ array(62, ),
        /* 167 */ array(64, ),
        /* 168 */ array(25, ),
        /* 169 */ array(25, ),
        /* 170 */ array(25, ),
        /* 171 */ array(25, ),
        /* 172 */ array(25, ),
        /* 173 */ array(25, ),
        /* 174 */ array(18, ),
        /* 175 */ array(25, ),
        /* 176 */ array(25, ),
        /* 177 */ array(25, ),
        /* 178 */ array(25, ),
        /* 179 */ array(25, ),
        /* 180 */ array(25, ),
        /* 181 */ array(25, ),
        /* 182 */ array(25, ),
        /* 183 */ array(25, ),
        /* 184 */ array(25, ),
        /* 185 */ array(25, ),
        /* 186 */ array(25, ),
        /* 187 */ array(25, ),
        /* 188 */ array(25, ),
        /* 189 */ array(25, ),
        /* 190 */ array(),
        /* 191 */ array(),
        /* 192 */ array(),
        /* 193 */ array(),
        /* 194 */ array(),
        /* 195 */ array(),
        /* 196 */ array(),
        /* 197 */ array(),
        /* 198 */ array(),
        /* 199 */ array(),
        /* 200 */ array(),
        /* 201 */ array(),
        /* 202 */ array(),
        /* 203 */ array(),
        /* 204 */ array(),
        /* 205 */ array(),
        /* 206 */ array(),
        /* 207 */ array(),
        /* 208 */ array(),
        /* 209 */ array(),
        /* 210 */ array(),
        /* 211 */ array(),
        /* 212 */ array(),
        /* 213 */ array(),
        /* 214 */ array(),
        /* 215 */ array(),
        /* 216 */ array(),
        /* 217 */ array(),
        /* 218 */ array(),
        /* 219 */ array(),
        /* 220 */ array(),
        /* 221 */ array(),
        /* 222 */ array(),
        /* 223 */ array(),
        /* 224 */ array(),
        /* 225 */ array(),
        /* 226 */ array(),
        /* 227 */ array(),
        /* 228 */ array(),
        /* 229 */ array(),
        /* 230 */ array(),
        /* 231 */ array(),
        /* 232 */ array(),
        /* 233 */ array(),
        /* 234 */ array(),
        /* 235 */ array(),
        /* 236 */ array(),
        /* 237 */ array(),
        /* 238 */ array(),
        /* 239 */ array(),
        /* 240 */ array(),
        /* 241 */ array(),
        /* 242 */ array(),
        /* 243 */ array(),
        /* 244 */ array(),
        /* 245 */ array(),
        /* 246 */ array(),
        /* 247 */ array(),
        /* 248 */ array(),
        /* 249 */ array(),
        /* 250 */ array(),
        /* 251 */ array(),
        /* 252 */ array(),
        /* 253 */ array(),
        /* 254 */ array(),
        /* 255 */ array(),
        /* 256 */ array(),
        /* 257 */ array(),
        /* 258 */ array(),
        /* 259 */ array(),
        /* 260 */ array(),
        /* 261 */ array(),
        /* 262 */ array(),
        /* 263 */ array(),
        /* 264 */ array(),
);
    static public $yy_default = array(
 /*     0 */   267,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    10 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    20 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    30 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    40 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    50 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    60 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    70 */   354,  354,  354,  354,  354,  354,  354,  354,  265,  354,
 /*    80 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    90 */   354,  354,  354,  354,  267,  267,  267,  267,  267,  267,
 /*   100 */   267,  267,  267,  267,  267,  267,  267,  267,  267,  267,
 /*   110 */   267,  267,  267,  267,  267,  267,  354,  354,  334,  292,
 /*   120 */   335,  336,  338,  320,  354,  354,  354,  354,  354,  354,
 /*   130 */   354,  354,  320,  354,  354,  354,  337,  351,  324,  316,
 /*   140 */   354,  354,  354,  354,  354,  339,  354,  354,  354,  354,
 /*   150 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*   160 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*   170 */   354,  354,  354,  354,  328,  354,  354,  354,  354,  354,
 /*   180 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*   190 */   313,  304,  349,  289,  271,  299,  314,  315,  350,  298,
 /*   200 */   270,  306,  296,  269,  297,  276,  277,  275,  274,  353,
 /*   210 */   273,  278,  279,  284,  285,  283,  282,  280,  281,  352,
 /*   220 */   348,  345,  325,  344,  272,  266,  268,  326,  327,  346,
 /*   230 */   347,  343,  333,  332,  286,  287,  301,  302,  300,  295,
 /*   240 */   293,  294,  303,  305,  310,  311,  309,  308,  307,  342,
 /*   250 */   341,  290,  291,  288,  323,  321,  322,  317,  319,  331,
 /*   260 */   340,  330,  329,  318,  312,
);
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    self::YYNOCODE      is a number which corresponds
**                        to no legal terminal or nonterminal number.  This
**                        number is used to fill in empty slots of the hash 
**                        table.
**    self::YYFALLBACK    If defined, this indicates that one or more tokens
**                        have fall-back values which should be used if the
**                        original value of the token will not parse.
**    self::YYSTACKDEPTH  is the maximum depth of the parser's stack.
**    self::YYNSTATE      the combined number of states.
**    self::YYNRULE       the number of rules in the grammar
**    self::YYERRORSYMBOL is the code number of the error symbol.  If not
**                        defined, then do no error processing.
*/
    const YYNOCODE = 94;
    const YYSTACKDEPTH = 100;
    const YYNSTATE = 265;
    const YYNRULE = 89;
    const YYERRORSYMBOL = 65;
    const YYERRSYMDT = 'yy0';
    const YYFALLBACK = 0;
    /** The next table maps tokens into fallback tokens.  If a construct
     * like the following:
     * 
     *      %fallback ID X Y Z.
     *
     * appears in the grammer, then ID becomes a fallback token for X, Y,
     * and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
     * but it does not parse, the type of the token is changed to ID and
     * the parse is retried before an error is thrown.
     */
    static public $yyFallback = array(
    );
    /**
     * Turn parser tracing on by giving a stream to which to write the trace
     * and a prompt to preface each trace message.  Tracing is turned off
     * by making either argument NULL 
     *
     * Inputs:
     * 
     * - A stream resource to which trace output should be written.
     *   If NULL, then tracing is turned off.
     * - A prefix string written at the beginning of every
     *   line of trace output.  If NULL, then tracing is
     *   turned off.
     *
     * Outputs:
     * 
     * - None.
     * @param resource
     * @param string
     */
    static function Trace($TraceFILE, $zTracePrompt)
    {
        if (!$TraceFILE) {
            $zTracePrompt = 0;
        } elseif (!$zTracePrompt) {
            $TraceFILE = 0;
        }
        self::$yyTraceFILE = $TraceFILE;
        self::$yyTracePrompt = $zTracePrompt;
    }

    /**
     * Output debug information to output (php://output stream)
     */
    static function PrintTrace()
    {
        self::$yyTraceFILE = fopen('php://output', 'w');
        self::$yyTracePrompt = '';
    }

    /**
     * @var resource|0
     */
    static public $yyTraceFILE;
    /**
     * String to prepend to debug output
     * @var string|0
     */
    static public $yyTracePrompt;
    /**
     * @var int
     */
    public $yyidx;                    /* Index of top element in stack */
    /**
     * @var int
     */
    public $yyerrcnt;                 /* Shifts left before out of the error */
    /**
     * @var array
     */
    public $yystack = array();  /* The parser's stack */

    /**
     * For tracing shifts, the names of all terminals and nonterminals
     * are required.  The following table supplies these names
     * @var array
     */
    static public $yyTokenName = array( 
  '$',             'T_TAG_OPEN',    'T_NOT',         'T_AND',       
  'T_OR',          'T_EQ',          'T_NE',          'T_GT',        
  'T_GE',          'T_LT',          'T_LE',          'T_IN',        
  'T_PLUS',        'T_MINUS',       'T_CONCAT',      'T_TIMES',     
  'T_DIV',         'T_MOD',         'T_PIPE',        'T_BITWISE',   
  'T_HTML',        'T_COMMENT',     'T_PRINT_OPEN',  'T_PRINT_CLOSE',
  'T_EXTENDS',     'T_TAG_CLOSE',   'T_INCLUDE',     'T_AUTOESCAPE',
  'T_CUSTOM_END',  'T_CUSTOM_TAG',  'T_AS',          'T_CUSTOM_BLOCK',
  'T_SPACEFULL',   'T_WITH',        'T_SET',         'T_ASSIGN',    
  'T_LOAD',        'T_FOR',         'T_COMMA',       'T_STEP',      
  'T_EMPTY',       'T_IF',          'T_ELSE',        'T_IFCHANGED', 
  'T_IFEQUAL',     'T_IFNOTEQUAL',  'T_BLOCK',       'T_FILTER',    
  'T_REGROUP',     'T_BY',          'T_COLON',       'T_TRUE',      
  'T_FALSE',       'T_STRING',      'T_INTL',        'T_RPARENT',   
  'T_LPARENT',     'T_OBJ',         'T_DOT',         'T_ALPHA',     
  'T_CLASS',       'T_BRACKETS_OPEN',  'T_BRACKETS_CLOSE',  'T_DOTDOT',    
  'T_NUMERIC',     'error',         'start',         'body',        
  'code',          'stmts',         'filtered_var',  'var_or_string',
  'stmt',          'for_stmt',      'ifchanged_stmt',  'block_stmt',  
  'filter_stmt',   'if_stmt',       'custom_tag',    'alias',       
  'ifequal',       'varname',       'params',        'expr',        
  'regroup',       'string',        'for_def',       'range',       
  'numvar',        'fvar_or_string',  'number',        'varname_args',
  'varpart',     
    );

    /**
     * For tracing reduce actions, the names of all rules are required.
     * @var array
     */
    static public $yyRuleName = array(
 /*   0 */ "start ::= body",
 /*   1 */ "body ::= body code",
 /*   2 */ "body ::=",
 /*   3 */ "code ::= T_TAG_OPEN stmts",
 /*   4 */ "code ::= T_HTML",
 /*   5 */ "code ::= T_COMMENT",
 /*   6 */ "code ::= T_PRINT_OPEN filtered_var T_PRINT_CLOSE",
 /*   7 */ "stmts ::= T_EXTENDS var_or_string T_TAG_CLOSE",
 /*   8 */ "stmts ::= stmt T_TAG_CLOSE",
 /*   9 */ "stmts ::= for_stmt",
 /*  10 */ "stmts ::= ifchanged_stmt",
 /*  11 */ "stmts ::= block_stmt",
 /*  12 */ "stmts ::= filter_stmt",
 /*  13 */ "stmts ::= if_stmt",
 /*  14 */ "stmts ::= T_INCLUDE var_or_string T_TAG_CLOSE",
 /*  15 */ "stmts ::= custom_tag",
 /*  16 */ "stmts ::= alias",
 /*  17 */ "stmts ::= ifequal",
 /*  18 */ "stmts ::= T_AUTOESCAPE varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  19 */ "custom_tag ::= T_CUSTOM_TAG T_TAG_CLOSE",
 /*  20 */ "custom_tag ::= T_CUSTOM_TAG T_AS varname T_TAG_CLOSE",
 /*  21 */ "custom_tag ::= T_CUSTOM_TAG params T_TAG_CLOSE",
 /*  22 */ "custom_tag ::= T_CUSTOM_TAG params T_AS varname T_TAG_CLOSE",
 /*  23 */ "custom_tag ::= T_CUSTOM_BLOCK T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  24 */ "custom_tag ::= T_CUSTOM_BLOCK params T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  25 */ "custom_tag ::= T_SPACEFULL T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  26 */ "alias ::= T_WITH varname T_AS varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  27 */ "stmt ::= T_SET varname T_ASSIGN expr",
 /*  28 */ "stmt ::= regroup",
 /*  29 */ "stmt ::= T_LOAD string",
 /*  30 */ "for_def ::= T_FOR varname T_IN filtered_var T_TAG_CLOSE",
 /*  31 */ "for_def ::= T_FOR varname T_COMMA varname T_IN filtered_var T_TAG_CLOSE",
 /*  32 */ "for_stmt ::= for_def body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  33 */ "for_stmt ::= T_FOR varname T_IN range T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  34 */ "for_stmt ::= T_FOR varname T_IN range T_STEP numvar T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  35 */ "for_stmt ::= for_def body T_TAG_OPEN T_EMPTY T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  36 */ "if_stmt ::= T_IF expr T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  37 */ "if_stmt ::= T_IF expr T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  38 */ "ifchanged_stmt ::= T_IFCHANGED T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  39 */ "ifchanged_stmt ::= T_IFCHANGED params T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  40 */ "ifchanged_stmt ::= T_IFCHANGED T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  41 */ "ifchanged_stmt ::= T_IFCHANGED params T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  42 */ "ifequal ::= T_IFEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  43 */ "ifequal ::= T_IFEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  44 */ "ifequal ::= T_IFNOTEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  45 */ "ifequal ::= T_IFNOTEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  46 */ "block_stmt ::= T_BLOCK varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  47 */ "block_stmt ::= T_BLOCK varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END varname T_TAG_CLOSE",
 /*  48 */ "block_stmt ::= T_BLOCK number T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  49 */ "block_stmt ::= T_BLOCK number T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END number T_TAG_CLOSE",
 /*  50 */ "filter_stmt ::= T_FILTER filtered_var T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  51 */ "regroup ::= T_REGROUP filtered_var T_BY varname T_AS varname",
 /*  52 */ "filtered_var ::= filtered_var T_PIPE varname_args",
 /*  53 */ "filtered_var ::= varname_args",
 /*  54 */ "varname_args ::= varname T_COLON var_or_string",
 /*  55 */ "varname_args ::= varname",
 /*  56 */ "params ::= params var_or_string",
 /*  57 */ "params ::= params T_COMMA var_or_string",
 /*  58 */ "params ::= var_or_string",
 /*  59 */ "var_or_string ::= varname",
 /*  60 */ "var_or_string ::= number",
 /*  61 */ "var_or_string ::= T_TRUE|T_FALSE",
 /*  62 */ "var_or_string ::= string",
 /*  63 */ "fvar_or_string ::= filtered_var",
 /*  64 */ "fvar_or_string ::= number",
 /*  65 */ "fvar_or_string ::= T_TRUE|T_FALSE",
 /*  66 */ "fvar_or_string ::= string",
 /*  67 */ "string ::= T_STRING",
 /*  68 */ "string ::= T_INTL T_STRING T_RPARENT",
 /*  69 */ "expr ::= T_NOT expr",
 /*  70 */ "expr ::= expr T_AND expr",
 /*  71 */ "expr ::= expr T_OR expr",
 /*  72 */ "expr ::= expr T_PLUS|T_MINUS|T_CONCAT expr",
 /*  73 */ "expr ::= expr T_EQ|T_NE|T_GT|T_GE|T_LT|T_LE|T_IN expr",
 /*  74 */ "expr ::= expr T_TIMES|T_DIV|T_MOD expr",
 /*  75 */ "expr ::= expr T_BITWISE|T_PIPE expr",
 /*  76 */ "expr ::= T_LPARENT expr T_RPARENT",
 /*  77 */ "expr ::= fvar_or_string",
 /*  78 */ "varname ::= varpart",
 /*  79 */ "varpart ::= varname T_OBJ|T_DOT T_ALPHA|T_CUSTOM_TAG|T_CUSTOM_BLOCK",
 /*  80 */ "varpart ::= varname T_CLASS T_ALPHA|T_CUSTOM_TAG|T_CUSTOM_BLOCK",
 /*  81 */ "varpart ::= varname T_BRACKETS_OPEN var_or_string T_BRACKETS_CLOSE",
 /*  82 */ "varpart ::= T_ALPHA",
 /*  83 */ "varpart ::= T_BLOCK|T_CUSTOM_TAG|T_CUSTOM_BLOCK",
 /*  84 */ "range ::= numvar T_DOTDOT numvar",
 /*  85 */ "numvar ::= number",
 /*  86 */ "numvar ::= varname",
 /*  87 */ "number ::= T_NUMERIC",
 /*  88 */ "number ::= T_MINUS T_NUMERIC",
    );

    /**
     * This function returns the symbolic name associated with a token
     * value.
     * @param int
     * @return string
     */
    function tokenName($tokenType)
    {
        if ($tokenType === 0) {
            return 'End of Input';
        }
        if ($tokenType > 0 && $tokenType < count(self::$yyTokenName)) {
            return self::$yyTokenName[$tokenType];
        } else {
            return "Unknown";
        }
    }

    /**
     * The following function deletes the value associated with a
     * symbol.  The symbol can be either a terminal or nonterminal.
     * @param int the symbol code
     * @param mixed the symbol's value
     */
    static function yy_destructor($yymajor, $yypminor)
    {
        switch ($yymajor) {
        /* Here is inserted the actions which take place when a
        ** terminal or non-terminal is destroyed.  This can happen
        ** when the symbol is popped from the stack during a
        ** reduce or during error processing or when a parser is 
        ** being destroyed before it is finished parsing.
        **
        ** Note: during a reduce, the only symbols destroyed are those
        ** which appear on the RHS of the rule, but which are not used
        ** inside the C code.
        */
            default:  break;   /* If no destructor action specified: do nothing */
        }
    }

    /**
     * Pop the parser's stack once.
     *
     * If there is a destructor routine associated with the token which
     * is popped from the stack, then call it.
     *
     * Return the major token number for the symbol popped.
     * @param Haanga_yyParser
     * @return int
     */
    function yy_pop_parser_stack()
    {
        if (!count($this->yystack)) {
            return;
        }
        $yytos = array_pop($this->yystack);
        if (self::$yyTraceFILE && $this->yyidx >= 0) {
            fwrite(self::$yyTraceFILE,
                self::$yyTracePrompt . 'Popping ' . self::$yyTokenName[$yytos->major] .
                    "\n");
        }
        $yymajor = $yytos->major;
        self::yy_destructor($yymajor, $yytos->minor);
        $this->yyidx--;
        return $yymajor;
    }

    /**
     * Deallocate and destroy a parser.  Destructors are all called for
     * all stack elements before shutting the parser down.
     */
    function __destruct()
    {
        while ($this->yyidx >= 0) {
            $this->yy_pop_parser_stack();
        }
        if (is_resource(self::$yyTraceFILE)) {
            fclose(self::$yyTraceFILE);
        }
    }

    /**
     * Based on the current state and parser stack, get a list of all
     * possible lookahead tokens
     * @param int
     * @return array
     */
    function yy_get_expected_tokens($token)
    {
        $state = $this->yystack[$this->yyidx]->stateno;
        $expected = self::$yyExpectedTokens[$state];
        if (in_array($token, self::$yyExpectedTokens[$state], true)) {
            return $expected;
        }
        $stack = $this->yystack;
        $yyidx = $this->yyidx;
        do {
            $yyact = $this->yy_find_shift_action($token);
            if ($yyact >= self::YYNSTATE && $yyact < self::YYNSTATE + self::YYNRULE) {
                // reduce action
                $done = 0;
                do {
                    if ($done++ == 100) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // too much recursion prevents proper detection
                        // so give up
                        return array_unique($expected);
                    }
                    $yyruleno = $yyact - self::YYNSTATE;
                    $this->yyidx -= self::$yyRuleInfo[$yyruleno]['rhs'];
                    $nextstate = $this->yy_find_reduce_action(
                        $this->yystack[$this->yyidx]->stateno,
                        self::$yyRuleInfo[$yyruleno]['lhs']);
                    if (isset(self::$yyExpectedTokens[$nextstate])) {
                        $expected += self::$yyExpectedTokens[$nextstate];
                            if (in_array($token,
                                  self::$yyExpectedTokens[$nextstate], true)) {
                            $this->yyidx = $yyidx;
                            $this->yystack = $stack;
                            return array_unique($expected);
                        }
                    }
                    if ($nextstate < self::YYNSTATE) {
                        // we need to shift a non-terminal
                        $this->yyidx++;
                        $x = new Haanga_yyStackEntry;
                        $x->stateno = $nextstate;
                        $x->major = self::$yyRuleInfo[$yyruleno]['lhs'];
                        $this->yystack[$this->yyidx] = $x;
                        continue 2;
                    } elseif ($nextstate == self::YYNSTATE + self::YYNRULE + 1) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // the last token was just ignored, we can't accept
                        // by ignoring input, this is in essence ignoring a
                        // syntax error!
                        return array_unique($expected);
                    } elseif ($nextstate === self::YY_NO_ACTION) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // input accepted, but not shifted (I guess)
                        return $expected;
                    } else {
                        $yyact = $nextstate;
                    }
                } while (true);
            }
            break;
        } while (true);
        return array_unique($expected);
    }

    /**
     * Based on the parser state and current parser stack, determine whether
     * the lookahead token is possible.
     * 
     * The parser will convert the token value to an error token if not.  This
     * catches some unusual edge cases where the parser would fail.
     * @param int
     * @return bool
     */
    function yy_is_expected_token($token)
    {
        if ($token === 0) {
            return true; // 0 is not part of this
        }
        $state = $this->yystack[$this->yyidx]->stateno;
        if (in_array($token, self::$yyExpectedTokens[$state], true)) {
            return true;
        }
        $stack = $this->yystack;
        $yyidx = $this->yyidx;
        do {
            $yyact = $this->yy_find_shift_action($token);
            if ($yyact >= self::YYNSTATE && $yyact < self::YYNSTATE + self::YYNRULE) {
                // reduce action
                $done = 0;
                do {
                    if ($done++ == 100) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // too much recursion prevents proper detection
                        // so give up
                        return true;
                    }
                    $yyruleno = $yyact - self::YYNSTATE;
                    $this->yyidx -= self::$yyRuleInfo[$yyruleno]['rhs'];
                    $nextstate = $this->yy_find_reduce_action(
                        $this->yystack[$this->yyidx]->stateno,
                        self::$yyRuleInfo[$yyruleno]['lhs']);
                    if (isset(self::$yyExpectedTokens[$nextstate]) &&
                          in_array($token, self::$yyExpectedTokens[$nextstate], true)) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        return true;
                    }
                    if ($nextstate < self::YYNSTATE) {
                        // we need to shift a non-terminal
                        $this->yyidx++;
                        $x = new Haanga_yyStackEntry;
                        $x->stateno = $nextstate;
                        $x->major = self::$yyRuleInfo[$yyruleno]['lhs'];
                        $this->yystack[$this->yyidx] = $x;
                        continue 2;
                    } elseif ($nextstate == self::YYNSTATE + self::YYNRULE + 1) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        if (!$token) {
                            // end of input: this is valid
                            return true;
                        }
                        // the last token was just ignored, we can't accept
                        // by ignoring input, this is in essence ignoring a
                        // syntax error!
                        return false;
                    } elseif ($nextstate === self::YY_NO_ACTION) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // input accepted, but not shifted (I guess)
                        return true;
                    } else {
                        $yyact = $nextstate;
                    }
                } while (true);
            }
            break;
        } while (true);
        $this->yyidx = $yyidx;
        $this->yystack = $stack;
        return true;
    }

    /**
     * Find the appropriate action for a parser given the terminal
     * look-ahead token iLookAhead.
     *
     * If the look-ahead token is YYNOCODE, then check to see if the action is
     * independent of the look-ahead.  If it is, return the action, otherwise
     * return YY_NO_ACTION.
     * @param int The look-ahead token
     */
    function yy_find_shift_action($iLookAhead)
    {
        $stateno = $this->yystack[$this->yyidx]->stateno;
     
        /* if ($this->yyidx < 0) return self::YY_NO_ACTION;  */
        if (!isset(self::$yy_shift_ofst[$stateno])) {
            // no shift actions
            return self::$yy_default[$stateno];
        }
        $i = self::$yy_shift_ofst[$stateno];
        if ($i === self::YY_SHIFT_USE_DFLT) {
            return self::$yy_default[$stateno];
        }
        if ($iLookAhead == self::YYNOCODE) {
            return self::YY_NO_ACTION;
        }
        $i += $iLookAhead;
        if ($i < 0 || $i >= self::YY_SZ_ACTTAB ||
              self::$yy_lookahead[$i] != $iLookAhead) {
            if (count(self::$yyFallback) && $iLookAhead < count(self::$yyFallback)
                   && ($iFallback = self::$yyFallback[$iLookAhead]) != 0) {
                if (self::$yyTraceFILE) {
                    fwrite(self::$yyTraceFILE, self::$yyTracePrompt . "FALLBACK " .
                        self::$yyTokenName[$iLookAhead] . " => " .
                        self::$yyTokenName[$iFallback] . "\n");
                }
                return $this->yy_find_shift_action($iFallback);
            }
            return self::$yy_default[$stateno];
        } else {
            return self::$yy_action[$i];
        }
    }

    /**
     * Find the appropriate action for a parser given the non-terminal
     * look-ahead token $iLookAhead.
     *
     * If the look-ahead token is self::YYNOCODE, then check to see if the action is
     * independent of the look-ahead.  If it is, return the action, otherwise
     * return self::YY_NO_ACTION.
     * @param int Current state number
     * @param int The look-ahead token
     */
    function yy_find_reduce_action($stateno, $iLookAhead)
    {
        /* $stateno = $this->yystack[$this->yyidx]->stateno; */

        if (!isset(self::$yy_reduce_ofst[$stateno])) {
            return self::$yy_default[$stateno];
        }
        $i = self::$yy_reduce_ofst[$stateno];
        if ($i == self::YY_REDUCE_USE_DFLT) {
            return self::$yy_default[$stateno];
        }
        if ($iLookAhead == self::YYNOCODE) {
            return self::YY_NO_ACTION;
        }
        $i += $iLookAhead;
        if ($i < 0 || $i >= self::YY_SZ_ACTTAB ||
              self::$yy_lookahead[$i] != $iLookAhead) {
            return self::$yy_default[$stateno];
        } else {
            return self::$yy_action[$i];
        }
    }

    /**
     * Perform a shift action.
     * @param int The new state to shift in
     * @param int The major token to shift in
     * @param mixed the minor token to shift in
     */
    function yy_shift($yyNewState, $yyMajor, $yypMinor)
    {
        $this->yyidx++;
        if ($this->yyidx >= self::YYSTACKDEPTH) {
            $this->yyidx--;
            if (self::$yyTraceFILE) {
                fprintf(self::$yyTraceFILE, "%sStack Overflow!\n", self::$yyTracePrompt);
            }
            while ($this->yyidx >= 0) {
                $this->yy_pop_parser_stack();
            }
            /* Here code is inserted which will execute if the parser
            ** stack ever overflows */
            return;
        }
        $yytos = new Haanga_yyStackEntry;
        $yytos->stateno = $yyNewState;
        $yytos->major = $yyMajor;
        $yytos->minor = $yypMinor;
        array_push($this->yystack, $yytos);
        if (self::$yyTraceFILE && $this->yyidx > 0) {
            fprintf(self::$yyTraceFILE, "%sShift %d\n", self::$yyTracePrompt,
                $yyNewState);
            fprintf(self::$yyTraceFILE, "%sStack:", self::$yyTracePrompt);
            for($i = 1; $i <= $this->yyidx; $i++) {
                fprintf(self::$yyTraceFILE, " %s",
                    self::$yyTokenName[$this->yystack[$i]->major]);
            }
            fwrite(self::$yyTraceFILE,"\n");
        }
    }

    /**
     * The following table contains information about every rule that
     * is used during the reduce.
     *
     * <pre>
     * array(
     *  array(
     *   int $lhs;         Symbol on the left-hand side of the rule
     *   int $nrhs;     Number of right-hand side symbols in the rule
     *  ),...
     * );
     * </pre>
     */
    static public $yyRuleInfo = array(
  array( 'lhs' => 66, 'rhs' => 1 ),
  array( 'lhs' => 67, 'rhs' => 2 ),
  array( 'lhs' => 67, 'rhs' => 0 ),
  array( 'lhs' => 68, 'rhs' => 2 ),
  array( 'lhs' => 68, 'rhs' => 1 ),
  array( 'lhs' => 68, 'rhs' => 1 ),
  array( 'lhs' => 68, 'rhs' => 3 ),
  array( 'lhs' => 69, 'rhs' => 3 ),
  array( 'lhs' => 69, 'rhs' => 2 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 3 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 7 ),
  array( 'lhs' => 78, 'rhs' => 2 ),
  array( 'lhs' => 78, 'rhs' => 4 ),
  array( 'lhs' => 78, 'rhs' => 3 ),
  array( 'lhs' => 78, 'rhs' => 5 ),
  array( 'lhs' => 78, 'rhs' => 6 ),
  array( 'lhs' => 78, 'rhs' => 7 ),
  array( 'lhs' => 78, 'rhs' => 6 ),
  array( 'lhs' => 79, 'rhs' => 9 ),
  array( 'lhs' => 72, 'rhs' => 4 ),
  array( 'lhs' => 72, 'rhs' => 1 ),
  array( 'lhs' => 72, 'rhs' => 2 ),
  array( 'lhs' => 86, 'rhs' => 5 ),
  array( 'lhs' => 86, 'rhs' => 7 ),
  array( 'lhs' => 73, 'rhs' => 5 ),
  array( 'lhs' => 73, 'rhs' => 9 ),
  array( 'lhs' => 73, 'rhs' => 11 ),
  array( 'lhs' => 73, 'rhs' => 9 ),
  array( 'lhs' => 77, 'rhs' => 7 ),
  array( 'lhs' => 77, 'rhs' => 11 ),
  array( 'lhs' => 74, 'rhs' => 6 ),
  array( 'lhs' => 74, 'rhs' => 7 ),
  array( 'lhs' => 74, 'rhs' => 10 ),
  array( 'lhs' => 74, 'rhs' => 11 ),
  array( 'lhs' => 80, 'rhs' => 8 ),
  array( 'lhs' => 80, 'rhs' => 12 ),
  array( 'lhs' => 80, 'rhs' => 8 ),
  array( 'lhs' => 80, 'rhs' => 12 ),
  array( 'lhs' => 75, 'rhs' => 7 ),
  array( 'lhs' => 75, 'rhs' => 8 ),
  array( 'lhs' => 75, 'rhs' => 7 ),
  array( 'lhs' => 75, 'rhs' => 8 ),
  array( 'lhs' => 76, 'rhs' => 7 ),
  array( 'lhs' => 84, 'rhs' => 6 ),
  array( 'lhs' => 70, 'rhs' => 3 ),
  array( 'lhs' => 70, 'rhs' => 1 ),
  array( 'lhs' => 91, 'rhs' => 3 ),
  array( 'lhs' => 91, 'rhs' => 1 ),
  array( 'lhs' => 82, 'rhs' => 2 ),
  array( 'lhs' => 82, 'rhs' => 3 ),
  array( 'lhs' => 82, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 85, 'rhs' => 1 ),
  array( 'lhs' => 85, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 81, 'rhs' => 1 ),
  array( 'lhs' => 92, 'rhs' => 3 ),
  array( 'lhs' => 92, 'rhs' => 3 ),
  array( 'lhs' => 92, 'rhs' => 4 ),
  array( 'lhs' => 92, 'rhs' => 1 ),
  array( 'lhs' => 92, 'rhs' => 1 ),
  array( 'lhs' => 87, 'rhs' => 3 ),
  array( 'lhs' => 88, 'rhs' => 1 ),
  array( 'lhs' => 88, 'rhs' => 1 ),
  array( 'lhs' => 90, 'rhs' => 1 ),
  array( 'lhs' => 90, 'rhs' => 2 ),
    );

    /**
     * The following table contains a mapping of reduce action to method name
     * that handles the reduction.
     * 
     * If a rule is not set, it has no handler.
     */
    static public $yyReduceMap = array(
        0 => 0,
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4,
        5 => 5,
        6 => 6,
        7 => 7,
        8 => 8,
        68 => 8,
        9 => 9,
        10 => 9,
        11 => 9,
        12 => 9,
        13 => 9,
        15 => 9,
        16 => 9,
        17 => 9,
        28 => 9,
        55 => 9,
        67 => 9,
        77 => 9,
        82 => 9,
        83 => 9,
        85 => 9,
        87 => 9,
        14 => 14,
        18 => 18,
        19 => 19,
        20 => 20,
        21 => 21,
        22 => 22,
        23 => 23,
        24 => 24,
        25 => 25,
        26 => 26,
        27 => 27,
        29 => 29,
        30 => 30,
        31 => 31,
        32 => 32,
        33 => 33,
        34 => 34,
        35 => 35,
        36 => 36,
        37 => 37,
        38 => 38,
        39 => 39,
        40 => 40,
        41 => 41,
        42 => 42,
        43 => 43,
        44 => 44,
        45 => 45,
        46 => 46,
        47 => 47,
        49 => 47,
        48 => 48,
        50 => 50,
        51 => 51,
        52 => 52,
        57 => 52,
        53 => 53,
        58 => 53,
        54 => 54,
        56 => 56,
        59 => 59,
        86 => 59,
        60 => 60,
        64 => 60,
        61 => 61,
        65 => 61,
        62 => 62,
        66 => 62,
        63 => 63,
        69 => 69,
        70 => 70,
        71 => 70,
        72 => 70,
        74 => 70,
        73 => 73,
        75 => 75,
        76 => 76,
        78 => 78,
        79 => 79,
        80 => 80,
        81 => 81,
        84 => 84,
        88 => 88,
    );
    /* Beginning here are the reduction cases.  A typical example
    ** follows:
    **  #line <lineno> <grammarfile>
    **   function yy_r0($yymsp){ ... }           // User supplied code
    **  #line <lineno> <thisfile>
    */
#line 80 "Parser.y"
    function yy_r0(){ $this->body = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1680 "Parser.php"
#line 82 "Parser.y"
    function yy_r1(){ $this->_retvalue=$this->yystack[$this->yyidx + -1]->minor; $this->_retvalue[] = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1683 "Parser.php"
#line 83 "Parser.y"
    function yy_r2(){ $this->_retvalue = array();     }
#line 1686 "Parser.php"
#line 86 "Parser.y"
    function yy_r3(){ if (count($this->yystack[$this->yyidx + 0]->minor)) $this->yystack[$this->yyidx + 0]->minor['line'] = $this->lex->getLine();  $this->_retvalue = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1689 "Parser.php"
#line 87 "Parser.y"
    function yy_r4(){
    $this->_retvalue = array('operation' => 'html', 'html' => $this->yystack[$this->yyidx + 0]->minor, 'line' => $this->lex->getLine() ); 
    }
#line 1694 "Parser.php"
#line 90 "Parser.y"
    function yy_r5(){
    $this->yystack[$this->yyidx + 0]->minor=rtrim($this->yystack[$this->yyidx + 0]->minor); $this->_retvalue = array('operation' => 'comment', 'comment' => $this->yystack[$this->yyidx + 0]->minor); 
    }
#line 1699 "Parser.php"
#line 93 "Parser.y"
    function yy_r6(){
    $this->_retvalue = array('operation' => 'print_var', 'variable' => $this->yystack[$this->yyidx + -1]->minor, 'line' => $this->lex->getLine() ); 
    }
#line 1704 "Parser.php"
#line 97 "Parser.y"
    function yy_r7(){ $this->_retvalue = array('operation' => 'base', $this->yystack[$this->yyidx + -1]->minor);     }
#line 1707 "Parser.php"
#line 98 "Parser.y"
    function yy_r8(){ $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor;     }
#line 1710 "Parser.php"
#line 99 "Parser.y"
    function yy_r9(){ $this->_retvalue = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1713 "Parser.php"
#line 104 "Parser.y"
    function yy_r14(){ $this->_retvalue = array('operation' => 'include', $this->yystack[$this->yyidx + -1]->minor);     }
#line 1716 "Parser.php"
#line 108 "Parser.y"
    function yy_r18(){ 
    $this->yystack[$this->yyidx + -5]->minor = strtolower($this->yystack[$this->yyidx + -5]->minor);
    if ($this->yystack[$this->yyidx + -5]->minor != 'on' && $this->yystack[$this->yyidx + -5]->minor != 'off') {
        $this->Error("Invalid autoescape param (".$this->yystack[$this->yyidx + -5]->minor."), it must be on or off");
    }
    if ($this->yystack[$this->yyidx + -1]->minor != "endautoescape") {
        $this->Error("Invalid close tag ".$this->yystack[$this->yyidx + -1]->minor.", it must be endautoescape");
    }
    $this->_retvalue = array('operation' => 'autoescape', 'value' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1728 "Parser.php"
#line 122 "Parser.y"
    function yy_r19(){
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -1]->minor, 'list'=>array()); 
    }
#line 1733 "Parser.php"
#line 125 "Parser.y"
    function yy_r20(){
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -3]->minor, 'as' => $this->yystack[$this->yyidx + -1]->minor, 'list'=>array()); 
    }
#line 1738 "Parser.php"
#line 128 "Parser.y"
    function yy_r21(){ 
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -2]->minor, 'list' => $this->yystack[$this->yyidx + -1]->minor); 
    }
#line 1743 "Parser.php"
#line 131 "Parser.y"
    function yy_r22(){
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -4]->minor, 'as' => $this->yystack[$this->yyidx + -1]->minor, 'list' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1748 "Parser.php"
#line 136 "Parser.y"
    function yy_r23(){
    if ('end'.$this->yystack[$this->yyidx + -5]->minor != $this->yystack[$this->yyidx + -1]->minor) { 
        $this->error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor); 
    } 
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'list' => array());
    }
#line 1756 "Parser.php"
#line 142 "Parser.y"
    function yy_r24(){
    if ('end'.$this->yystack[$this->yyidx + -6]->minor != $this->yystack[$this->yyidx + -1]->minor) { 
        $this->error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor); 
    } 
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -6]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'list' => $this->yystack[$this->yyidx + -5]->minor);
    }
#line 1764 "Parser.php"
#line 150 "Parser.y"
    function yy_r25(){
    if ('endspacefull' != $this->yystack[$this->yyidx + -1]->minor) {
        $this->error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor);
    } 
    $this->_retvalue = array('operation' => 'spacefull', 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1772 "Parser.php"
#line 158 "Parser.y"
    function yy_r26(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endwith") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endwith");
    }
    $this->_retvalue = array('operation' => 'alias', 'var' => $this->yystack[$this->yyidx + -7]->minor, 'as' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1780 "Parser.php"
#line 166 "Parser.y"
    function yy_r27(){ $this->_retvalue = array('operation' => 'set', 'var' => $this->yystack[$this->yyidx + -2]->minor,'expr' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1783 "Parser.php"
#line 168 "Parser.y"
    function yy_r29(){
    if (!is_file($this->yystack[$this->yyidx + 0]->minor) || !Haanga_Compiler::getOption('enable_load')) {
        $this->error($this->yystack[$this->yyidx + 0]->minor." is not a valid file"); 
    } 
    require_once $this->yystack[$this->yyidx + 0]->minor;
    }
#line 1791 "Parser.php"
#line 177 "Parser.y"
    function yy_r30(){
    $var = $this->compiler->get_context($this->yystack[$this->yyidx + -1]->minor[0]);
    if (is_array($var) || $var instanceof Iterator) {
        /* let's check if it is an object or array */
        $this->compiler->set_context($this->yystack[$this->yyidx + -3]->minor, current($var));
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -3]->minor, 'index' => NULL, 'array' => $this->yystack[$this->yyidx + -1]->minor);
    }
#line 1801 "Parser.php"
#line 185 "Parser.y"
    function yy_r31(){
    $var = $this->compiler->get_context($this->yystack[$this->yyidx + -1]->minor[0]);
    if (is_array($var) || $var instanceof Iterator) {
        /* let's check if it is an object or array */
        $this->compiler->set_context($this->yystack[$this->yyidx + -3]->minor, current($var));
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -3]->minor, 'index' => $this->yystack[$this->yyidx + -5]->minor, 'array' => $this->yystack[$this->yyidx + -1]->minor);

    }
#line 1812 "Parser.php"
#line 195 "Parser.y"
    function yy_r32(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = $this->yystack[$this->yyidx + -4]->minor;
    $this->_retvalue['body'] = $this->yystack[$this->yyidx + -3]->minor;
    }
#line 1821 "Parser.php"
#line 203 "Parser.y"
    function yy_r33(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -7]->minor, 'range' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'variable' => $this->yystack[$this->yyidx + -7]->minor, 'step' => 1);
    }
#line 1829 "Parser.php"
#line 210 "Parser.y"
    function yy_r34(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -9]->minor, 'range' => $this->yystack[$this->yyidx + -7]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'variable' => $this->yystack[$this->yyidx + -9]->minor, 'step' => $this->yystack[$this->yyidx + -5]->minor);
    }
#line 1837 "Parser.php"
#line 217 "Parser.y"
    function yy_r35(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = $this->yystack[$this->yyidx + -8]->minor;
    $this->_retvalue['body']  = $this->yystack[$this->yyidx + -7]->minor;
    $this->_retvalue['empty'] = $this->yystack[$this->yyidx + -3]->minor;
    }
#line 1847 "Parser.php"
#line 226 "Parser.y"
    function yy_r36(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endif") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endif");
    }
    $this->_retvalue = array('operation' => 'if', 'expr' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1855 "Parser.php"
#line 232 "Parser.y"
    function yy_r37(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endif") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endif");
    }
    $this->_retvalue = array('operation' => 'if', 'expr' => $this->yystack[$this->yyidx + -9]->minor, 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1863 "Parser.php"
#line 240 "Parser.y"
    function yy_r38(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1871 "Parser.php"
#line 247 "Parser.y"
    function yy_r39(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -3]->minor, 'check' => $this->yystack[$this->yyidx + -5]->minor);
    }
#line 1879 "Parser.php"
#line 253 "Parser.y"
    function yy_r40(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1887 "Parser.php"
#line 260 "Parser.y"
    function yy_r41(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -7]->minor, 'check' => $this->yystack[$this->yyidx + -9]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1895 "Parser.php"
#line 268 "Parser.y"
    function yy_r42(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '==', 1 => $this->yystack[$this->yyidx + -6]->minor, 2 => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1903 "Parser.php"
#line 274 "Parser.y"
    function yy_r43(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '==', 1 => $this->yystack[$this->yyidx + -10]->minor, 2 => $this->yystack[$this->yyidx + -9]->minor, 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1911 "Parser.php"
#line 280 "Parser.y"
    function yy_r44(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifnotequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifnotequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '!=', 1 => $this->yystack[$this->yyidx + -6]->minor, 2 => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1919 "Parser.php"
#line 286 "Parser.y"
    function yy_r45(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifnotequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifnotequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '!=', 1 => $this->yystack[$this->yyidx + -10]->minor, 2 => $this->yystack[$this->yyidx + -9]->minor, 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1927 "Parser.php"
#line 294 "Parser.y"
    function yy_r46(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endblock") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endblock");
    }
    $this->_retvalue = array('operation' => 'block', 'name' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1935 "Parser.php"
#line 301 "Parser.y"
    function yy_r47(){
    if ($this->yystack[$this->yyidx + -2]->minor != "endblock") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -2]->minor.", expecting endblock");
    }
    $this->_retvalue = array('operation' => 'block', 'name' => $this->yystack[$this->yyidx + -6]->minor, 'body' => $this->yystack[$this->yyidx + -4]->minor); 
    }
#line 1943 "Parser.php"
#line 308 "Parser.y"
    function yy_r48(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endblock") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endblock");
    }
    $this->_retvalue = array('operation' => 'block', 'name' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1951 "Parser.php"
#line 323 "Parser.y"
    function yy_r50(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endfilter") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfilter");
    }
    $this->_retvalue = array('operation' => 'filter', 'functions' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1959 "Parser.php"
#line 331 "Parser.y"
    function yy_r51(){ $this->_retvalue=array('operation' => 'regroup', 'array' => $this->yystack[$this->yyidx + -4]->minor, 'row' => $this->yystack[$this->yyidx + -2]->minor, 'as' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1962 "Parser.php"
#line 334 "Parser.y"
    function yy_r52(){ $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor; $this->_retvalue[] = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1965 "Parser.php"
#line 335 "Parser.y"
    function yy_r53(){ $this->_retvalue = array($this->yystack[$this->yyidx + 0]->minor);     }
#line 1968 "Parser.php"
#line 337 "Parser.y"
    function yy_r54(){ $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor, 'args'=>array($this->yystack[$this->yyidx + 0]->minor));     }
#line 1971 "Parser.php"
#line 341 "Parser.y"
    function yy_r56(){ $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor; $this->_retvalue[] = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1974 "Parser.php"
#line 347 "Parser.y"
    function yy_r59(){ $this->_retvalue = array('var' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1977 "Parser.php"
#line 348 "Parser.y"
    function yy_r60(){ $this->_retvalue = array('number' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1980 "Parser.php"
#line 349 "Parser.y"
    function yy_r61(){ $this->_retvalue = trim(@$this->yystack[$this->yyidx + 0]->minor);     }
#line 1983 "Parser.php"
#line 350 "Parser.y"
    function yy_r62(){ $this->_retvalue = array('string' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1986 "Parser.php"
#line 353 "Parser.y"
    function yy_r63(){ $this->_retvalue = array('var_filter' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1989 "Parser.php"
#line 363 "Parser.y"
    function yy_r69(){ $this->_retvalue = array('op_expr' => 'not', $this->yystack[$this->yyidx + 0]->minor);     }
#line 1992 "Parser.php"
#line 364 "Parser.y"
    function yy_r70(){ $this->_retvalue = array('op_expr' => @$this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor);     }
#line 1995 "Parser.php"
#line 367 "Parser.y"
    function yy_r73(){ $this->_retvalue = array('op_expr' => trim(@$this->yystack[$this->yyidx + -1]->minor), $this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor);     }
#line 1998 "Parser.php"
#line 369 "Parser.y"
    function yy_r75(){ $this->_retvalue = array('op_expr' => 'expr', array('op_expr' => @$this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor));     }
#line 2001 "Parser.php"
#line 370 "Parser.y"
    function yy_r76(){ $this->_retvalue = array('op_expr' => 'expr', $this->yystack[$this->yyidx + -1]->minor);     }
#line 2004 "Parser.php"
#line 375 "Parser.y"
    function yy_r78(){ $this->_retvalue = current($this->compiler->generate_variable_name($this->yystack[$this->yyidx + 0]->minor, false));     }
#line 2007 "Parser.php"
#line 376 "Parser.y"
    function yy_r79(){ 
    if (!is_array($this->yystack[$this->yyidx + -2]->minor)) { $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor); } 
    else { $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor; }  $this->_retvalue[]=array('object' => $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2013 "Parser.php"
#line 380 "Parser.y"
    function yy_r80(){ 
    if (!is_array($this->yystack[$this->yyidx + -2]->minor)) { $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor); } 
    else { $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor; }  $this->_retvalue[]=array('class' => '$'.$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2019 "Parser.php"
#line 384 "Parser.y"
    function yy_r81(){
    if (!is_array($this->yystack[$this->yyidx + -3]->minor)) { $this->_retvalue = array($this->yystack[$this->yyidx + -3]->minor); } 
    else { $this->_retvalue = $this->yystack[$this->yyidx + -3]->minor; }  $this->_retvalue[]=$this->yystack[$this->yyidx + -1]->minor;
    }
#line 2025 "Parser.php"
#line 392 "Parser.y"
    function yy_r84(){ $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor);     }
#line 2028 "Parser.php"
#line 398 "Parser.y"
    function yy_r88(){ $this->_retvalue = -1 * ($this->yystack[$this->yyidx + 0]->minor);     }
#line 2031 "Parser.php"

    /**
     * placeholder for the left hand side in a reduce operation.
     * 
     * For a parser with a rule like this:
     * <pre>
     * rule(A) ::= B. { A = 1; }
     * </pre>
     * 
     * The parser will translate to something like:
     * 
     * <code>
     * function yy_r0(){$this->_retvalue = 1;}
     * </code>
     */
    private $_retvalue;

    /**
     * Perform a reduce action and the shift that must immediately
     * follow the reduce.
     * 
     * For a rule such as:
     * 
     * <pre>
     * A ::= B blah C. { dosomething(); }
     * </pre>
     * 
     * This function will first call the action, if any, ("dosomething();" in our
     * example), and then it will pop three states from the stack,
     * one for each entry on the right-hand side of the expression
     * (B, blah, and C in our example rule), and then push the result of the action
     * back on to the stack with the resulting state reduced to (as described in the .out
     * file)
     * @param int Number of the rule by which to reduce
     */
    function yy_reduce($yyruleno)
    {
        //int $yygoto;                     /* The next state */
        //int $yyact;                      /* The next action */
        //mixed $yygotominor;        /* The LHS of the rule reduced */
        //Haanga_yyStackEntry $yymsp;            /* The top of the parser's stack */
        //int $yysize;                     /* Amount to pop the stack */
        $yymsp = $this->yystack[$this->yyidx];
        if (self::$yyTraceFILE && $yyruleno >= 0 
              && $yyruleno < count(self::$yyRuleName)) {
            fprintf(self::$yyTraceFILE, "%sReduce (%d) [%s].\n",
                self::$yyTracePrompt, $yyruleno,
                self::$yyRuleName[$yyruleno]);
        }

        $this->_retvalue = $yy_lefthand_side = null;
        if (array_key_exists($yyruleno, self::$yyReduceMap)) {
            // call the action
            $this->_retvalue = null;
            $this->{'yy_r' . self::$yyReduceMap[$yyruleno]}();
            $yy_lefthand_side = $this->_retvalue;
        }
        $yygoto = self::$yyRuleInfo[$yyruleno]['lhs'];
        $yysize = self::$yyRuleInfo[$yyruleno]['rhs'];
        $this->yyidx -= $yysize;
        for($i = $yysize; $i; $i--) {
            // pop all of the right-hand side parameters
            array_pop($this->yystack);
        }
        $yyact = $this->yy_find_reduce_action($this->yystack[$this->yyidx]->stateno, $yygoto);
        if ($yyact < self::YYNSTATE) {
            /* If we are not debugging and the reduce action popped at least
            ** one element off the stack, then we can push the new element back
            ** onto the stack here, and skip the stack overflow test in yy_shift().
            ** That gives a significant speed improvement. */
            if (!self::$yyTraceFILE && $yysize) {
                $this->yyidx++;
                $x = new Haanga_yyStackEntry;
                $x->stateno = $yyact;
                $x->major = $yygoto;
                $x->minor = $yy_lefthand_side;
                $this->yystack[$this->yyidx] = $x;
            } else {
                $this->yy_shift($yyact, $yygoto, $yy_lefthand_side);
            }
        } elseif ($yyact == self::YYNSTATE + self::YYNRULE + 1) {
            $this->yy_accept();
        }
    }

    /**
     * The following code executes when the parse fails
     * 
     * Code from %parse_fail is inserted here
     */
    function yy_parse_failed()
    {
        if (self::$yyTraceFILE) {
            fprintf(self::$yyTraceFILE, "%sFail!\n", self::$yyTracePrompt);
        }
        while ($this->yyidx >= 0) {
            $this->yy_pop_parser_stack();
        }
        /* Here code is inserted which will be executed whenever the
        ** parser fails */
    }

    /**
     * The following code executes when a syntax error first occurs.
     * 
     * %syntax_error code is inserted here
     * @param int The major type of the error token
     * @param mixed The minor type of the error token
     */
    function yy_syntax_error($yymajor, $TOKEN)
    {
#line 71 "Parser.y"

    $expect = array();
    foreach ($this->yy_get_expected_tokens($yymajor) as $token) {
        $expect[] = self::$yyTokenName[$token];
    }
    $this->Error('Unexpected ' . $this->tokenName($yymajor) . '(' . $TOKEN. '), expected one of: ' . implode(',', $expect));
#line 2151 "Parser.php"
    }

    /**
     * The following is executed when the parser accepts
     * 
     * %parse_accept code is inserted here
     */
    function yy_accept()
    {
        if (self::$yyTraceFILE) {
            fprintf(self::$yyTraceFILE, "%sAccept!\n", self::$yyTracePrompt);
        }
        while ($this->yyidx >= 0) {
            $stack = $this->yy_pop_parser_stack();
        }
        /* Here code is inserted which will be executed whenever the
        ** parser accepts */
#line 57 "Parser.y"

#line 2172 "Parser.php"
    }

    /**
     * The main parser program.
     * 
     * The first argument is the major token number.  The second is
     * the token value string as scanned from the input.
     *
     * @param int the token number
     * @param mixed the token value
     * @param mixed any extra arguments that should be passed to handlers
     */
    function doParse($yymajor, $yytokenvalue)
    {
//        $yyact;            /* The parser action. */
//        $yyendofinput;     /* True if we are at the end of input */
        $yyerrorhit = 0;   /* True if yymajor has invoked an error */
        
        /* (re)initialize the parser, if necessary */
        if ($this->yyidx === null || $this->yyidx < 0) {
            /* if ($yymajor == 0) return; // not sure why this was here... */
            $this->yyidx = 0;
            $this->yyerrcnt = -1;
            $x = new Haanga_yyStackEntry;
            $x->stateno = 0;
            $x->major = 0;
            $this->yystack = array();
            array_push($this->yystack, $x);
        }
        $yyendofinput = ($yymajor==0);
        
        if (self::$yyTraceFILE) {
            fprintf(self::$yyTraceFILE, "%sInput %s\n",
                self::$yyTracePrompt, self::$yyTokenName[$yymajor]);
        }
        
        do {
            $yyact = $this->yy_find_shift_action($yymajor);
            if ($yymajor < self::YYERRORSYMBOL &&
                  !$this->yy_is_expected_token($yymajor)) {
                // force a syntax error
                $yyact = self::YY_ERROR_ACTION;
            }
            if ($yyact < self::YYNSTATE) {
                $this->yy_shift($yyact, $yymajor, $yytokenvalue);
                $this->yyerrcnt--;
                if ($yyendofinput && $this->yyidx >= 0) {
                    $yymajor = 0;
                } else {
                    $yymajor = self::YYNOCODE;
                }
            } elseif ($yyact < self::YYNSTATE + self::YYNRULE) {
                $this->yy_reduce($yyact - self::YYNSTATE);
            } elseif ($yyact == self::YY_ERROR_ACTION) {
                if (self::$yyTraceFILE) {
                    fprintf(self::$yyTraceFILE, "%sSyntax Error!\n",
                        self::$yyTracePrompt);
                }
                if (self::YYERRORSYMBOL) {
                    /* A syntax error has occurred.
                    ** The response to an error depends upon whether or not the
                    ** grammar defines an error token "ERROR".  
                    **
                    ** This is what we do if the grammar does define ERROR:
                    **
                    **  * Call the %syntax_error function.
                    **
                    **  * Begin popping the stack until we enter a state where
                    **    it is legal to shift the error symbol, then shift
                    **    the error symbol.
                    **
                    **  * Set the error count to three.
                    **
                    **  * Begin accepting and shifting new tokens.  No new error
                    **    processing will occur until three tokens have been
                    **    shifted successfully.
                    **
                    */
                    if ($this->yyerrcnt < 0) {
                        $this->yy_syntax_error($yymajor, $yytokenvalue);
                    }
                    $yymx = $this->yystack[$this->yyidx]->major;
                    if ($yymx == self::YYERRORSYMBOL || $yyerrorhit ){
                        if (self::$yyTraceFILE) {
                            fprintf(self::$yyTraceFILE, "%sDiscard input token %s\n",
                                self::$yyTracePrompt, self::$yyTokenName[$yymajor]);
                        }
                        $this->yy_destructor($yymajor, $yytokenvalue);
                        $yymajor = self::YYNOCODE;
                    } else {
                        while ($this->yyidx >= 0 &&
                                 $yymx != self::YYERRORSYMBOL &&
        ($yyact = $this->yy_find_shift_action(self::YYERRORSYMBOL)) >= self::YYNSTATE
                              ){
                            $this->yy_pop_parser_stack();
                        }
                        if ($this->yyidx < 0 || $yymajor==0) {
                            $this->yy_destructor($yymajor, $yytokenvalue);
                            $this->yy_parse_failed();
                            $yymajor = self::YYNOCODE;
                        } elseif ($yymx != self::YYERRORSYMBOL) {
                            $u2 = 0;
                            $this->yy_shift($yyact, self::YYERRORSYMBOL, $u2);
                        }
                    }
                    $this->yyerrcnt = 3;
                    $yyerrorhit = 1;
                } else {
                    /* YYERRORSYMBOL is not defined */
                    /* This is what we do if the grammar does not define ERROR:
                    **
                    **  * Report an error message, and throw away the input token.
                    **
                    **  * If the input token is $, then fail the parse.
                    **
                    ** As before, subsequent error messages are suppressed until
                    ** three input tokens have been successfully shifted.
                    */
                    if ($this->yyerrcnt <= 0) {
                        $this->yy_syntax_error($yymajor, $yytokenvalue);
                    }
                    $this->yyerrcnt = 3;
                    $this->yy_destructor($yymajor, $yytokenvalue);
                    if ($yyendofinput) {
                        $this->yy_parse_failed();
                    }
                    $yymajor = self::YYNOCODE;
                }
            } else {
                $this->yy_accept();
                $yymajor = self::YYNOCODE;
            }            
        } while ($yymajor != self::YYNOCODE && $this->yyidx >= 0);
    }
}